import java.math.BigInteger;
import java.lang.ArithmeticException;
import java.util.*;

/**
 * In dieser Klasse wird der Angriff auf die direkte RSA-Implementierung
 * von Mixen implementiert.
 */

public class Attack {

    
    /* Fuer Internationalisierung */
    public static ResourceBundle messages; 
    
    /**
     * Der anzugreifende Mix
     */
    
    private Mix mix;

    /**
     * Message Tools für den Mix
     */
    
    private MessageTools messageTools;
    
    /**
     * Hauptfunktion, in der auch der Angriff implementiert wird.
     */
    
    public Attack() {
	/* Mix erzeugen */
	
	mix = new Mix();
	messageTools = mix.getMessageTools();
	// Start
	System.out.println(messages.getString("AttackAngMix"));
	System.out.println(messages.getString("AttackLen")+" "+Integer.toString(mix.getMessageLength()));
	System.out.println(messages.getString("AttackRan")+" "+Integer.toString(mix.getRandomLength()));

	/* Hinweise:
	   Zur Implementierung des des Angriffs können alle BigInteger-Methoden,
	   die Methoden des im Objekt "mix" bereitgestellten und bereits initialisierten Mixes
	   und die Methode minAbs(), die die betragskleinste Repräsentation eines Wertes v (mod n)
	   angibt, verwendet werden.
	   
	   Ein- und Ausgabenachrichten müssen nicht gesondert gespeichert werden, dies tut
	   bereits der Mix. Sie können mit den Methoden getInBatch() bzw. getOutBatch() gelesen werden.
	   (siehe auch Java-Doku zu diesen Methoden)
	   Im Angriff genügt es deshalb, sich jeweils nur Positionen der Nachrichten im Schub zu
	   merken.

	   Mit der Mix-Methode addInMessage() können Eingabenachrichten zum nächsten Schub des
	   Mixes hinzugefügt werden.
	   Um die Bearbeitung eines Schubes im Mix auszulösen, muß die Mix-Methode executeBatch()
	   aufgerufen werden. Diese füllt den Schub zunächst mit zufälligen Nachrichten auf, und
	   führt dann die Mixoperationen durch.
	   
	   Mit der Methode des Mixes checkAttackSuccess() kann nach Beendigung des Angriffs
	   getestet werden, ob die ermittelte Nachrichtenzuordnung korrekt ist. Diese Methode darf
	   für die Implementierung des Angriffes selbst natürlich nicht verwendet werden ;-)
	   
	   Weiterhin können folgende Angaben aus dem Mix gelesen werden:
	   Schubgröße:                       mix.BATCH_SIZE
	   Nachrichtenlänge L:               mix.getMessageLength
	   Länge b der Zufallsbitkette Z:    mix.getRandomLength
	   Der Modulus des Mix-RSA-Keys:     mix.getPublicKey().n
	*/
	
	/* ersten Schub (Schubnummer 0) vom Mix bearbeiten lassen */
	
	mix.executeBatch();

	//------------------------------------------------------
	/* Hier implementieren                                  */
	
	//------------------------------------------------------

	/* Schematische Darstellung des Angriffs
	 -------------------------------------
	 wähle Nachricht aus, die aufgedeckt werden soll

	 boolean found = false;
	 do{

	   ... Berechnungen für Angriff

	   mix.executeBatch();             // Mix nächsten Schub bearbeiten lassen

	   ... weitere Berechnungen für Angriff

	 }while (!found);                  // bis Nachrichtenzuordnung ermittelt wurde

	 ... Test mittels mix.checkAttackSuccess(), ob Zuordnung korrekt ist

	 ... Zuordnung ausgeben
	*/

	



	
	
	


	//---------------------------------------------------------
	/* bis hier implementieren                               */
	//---------------------------------------------------------
	
    }
    
    /**
     * Berechnet die betragskleinste Repräsentation des Wertes v (mod n)
     * @param v der Wert
     * @param n der Modulus
     * @return die betragskleinste Repräsentation des Wertes v (mod n)
     * @exception ArithmeticException wenn n <= 0
     */
    
    private BigInteger minAbs(BigInteger v, BigInteger n) throws ArithmeticException{
	if(n.compareTo(BigInteger.ZERO) <= 0){
	    throw new ArithmeticException(messages.getString("AttackminAbs"));
	}
	v = v.mod(n);
	if(v.compareTo(n.divide(new BigInteger("2"))) > 0){
	    v = v.subtract(n);
	}
	return v;
    }
    
    /**
     * Methode zum Testen der Funktion des Mixes. Diese wird im Praktikum nicht benötigt.
     */
    
    private void testMix(){
	System.out.println("XXXXXXXXXXXXX MixTest XXXXXXXXXXXX");
	for(int i = 0; i < 10; i++){
	    mix.executeBatch();
	    System.out.println(messages.getString("AttackBat")+" "+Integer.toString(i));
	    System.out.println(messages.getString("AttackAusg"));
	    for(int j = 0; j < Mix.BATCH_SIZE; j++){
		System.out.println("'"+messageTools.decodeMessage(mix.getOutBatch(i)[j])+"'");
	    }
	}
    }
    
    /**
     * main-Methode, zum starten des Angriffs
     */
    
    public static void main(String[] args) {
	
	
	/* Der folgende Teil ist nötig für die Internationalisierung*/
	Locale currentLocale;
	currentLocale = Locale.getDefault();
	messages = ResourceBundle.getBundle("localmessages",currentLocale);

	
	/* Main-Methode */
	Attack attack = new Attack();
    }
    
    
}
