
import java.math.BigInteger;

import java.util.Random;
/**
 * Diese Klassen implementiert Tools zur Nachrichtenbehandlung für einen Mix.
 * @author Sebastian Clauß
 * @version 1.0
 */

public class MessageTools {

  /**
   * Nachrichtenlänge
   */
  
  private int messageLength;
  /**
   * Länge der Zufallsbitkette in einer Nachricht
   */
  
  private int randomLength;
  /**
   * Zufallsgenerator
   */
  
    Random random = new Random();  /* hier könnte auch ein kryptographisch starker genommen werden */
  /**
   * Der öffentliche Schlüssel des Mixes
   */
  
  private RSAPublicKey mixPublicKey;

  /**
   * Initialisiert die Tools mit den Parametern eines Mixes.
   * @param mix der Mix, mit dessen Parametern die Tools initialisiert werden sollen.
   */
  
  public MessageTools(Mix mix){
    messageLength = mix.getMessageLength();
    randomLength = mix.getRandomLength();
    mixPublicKey = mix.getPublicKey();
  }


  /**
   * generieren einer Nachricht aus einer angegebenen Zeichenkette.
   * Vom Text werden nur soviele Zeichen verwendet,
   * wie in eine Nachricht passen.
   * @param message der Inhaltstext der Nachricht
   * @return die für den Mix aufbereitete Nachricht
   */
  
  public BigInteger generateMessage(String message){
    BigInteger mixMessage = new BigInteger("0");
    if(messageLength <= 0){
      return BigInteger.ZERO;
    }else{
      if(randomLength > 0){
        mixMessage = mixMessage.add(new BigInteger(randomLength, random))
                               .shiftLeft(messageLength-randomLength);
      }
      /* Nachricht in BigInteger umwandeln und Länge bei Bedarf einschränken */
      
      int charNumber = (messageLength - randomLength) / 8;
      byte msgChars[] = new byte[charNumber];
      byte msgChars2[] = message.getBytes();
      for(int i = 0; i < charNumber; i++){
        if(i < message.getBytes().length){
          msgChars[i] = msgChars2[i];
        }else{
          msgChars[i] = ' ';
        }
      }

      mixMessage = mixMessage.add(new BigInteger(msgChars));

      /* Nachricht verschlüsseln*/
      
      return RSA.encrypt(mixPublicKey, mixMessage);
    }
  }

  /**
   * Erzeugen einer zufälligen Nachricht für einen Mix.
   * @return eine zufällige Nachricht
   */
  
  public BigInteger generateRandomMessage(){
    return RSA.encrypt(mixPublicKey, new BigInteger(messageLength, random));
  };

  /**
   * Decodieren einer Nachricht, die der Mix ausgegeben hat.
   * @param message die Nachricht im BigInteger-Format
   * @return den Text in der Nachricht
   */
  
  public String decodeMessage(BigInteger message){
    byte msgChars[] = message.toByteArray();
    return new String(msgChars);
  }


}
