
import java.math.BigInteger;
import algorithms.*;

/**
 * Diese Klasse implementiert das asymmetrische Verschlüsselungssystem RSA (pur).
 */

public class RSA{

    /**
     * Diese Methode implementiert die Entschlüsselungsfunktion von RSA.
     * Die Nachricht m liegt dabei schon im BigInteger-Format vor.
     * Vereinfachend kann hier davon ausgegangen werden, daß m < n ist.
     *
     * Es konnen die BigInteger-Methoden und zusätzlich die
     * Algorithmen CRA, Legendre und SQRTMOD verwendet werden.
     *
     * @param privKey der geheime Entschlüsselungsschlüssel
     * @param ciph die Nachricht
     * @return der Klartext
     */
    
    
    public static BigInteger decrypt(RSAPrivateKey privKey, BigInteger ciph){
	try{

	    /* Beschleunigte Signierfunktion */
	    
	    BigInteger clearP = ciph.modPow(privKey.dp, privKey.p);   // m^(e^-1 mod p-1) mod p
	    BigInteger clearQ = ciph.modPow(privKey.dq, privKey.q);   /* das gleiche mit q */

	    BigInteger reste[] = {clearP,clearQ};                     /* clearP und clearQ mit dem CRA */
	    BigInteger mods[] = {privKey.p,privKey.q};                /* zusammenfassen */
	    CRA myCRA = new CRA(reste,mods);
	    myCRA.executeCRA();
	    return myCRA.getErgebnis();                               /* entschlüsselten Text zurückgeben */
	  }catch (Exception e){
	    System.out.println("RSA.sign: "+e.getMessage());
	    e.printStackTrace();
	  }
	  return null;
  }

    /**
     * Diese Methode implementiert die Verschlüsselungsfunktion von RSA.
     *
     * @param pubKey der öffentliche Schlüssel
     * @param m die Nachricht
     * @return der Schlüsseltext in Form eines BigIntegers
     */
    
    public static BigInteger encrypt(RSAPublicKey pubKey, BigInteger m){

        return m.modPow(pubKey.e, pubKey.n);  // ciph = m^e mod n
    }
}
